/* Copyright (c) 2015 Nordic Semiconductor. All Rights Reserved.
 *
 * The information contained herein is property of Nordic Semiconductor ASA.
 * Terms and conditions of usage are described in detail in NORDIC
 * SEMICONDUCTOR STANDARD SOFTWARE LICENSE AGREEMENT.
 *
 * Licensees are granted free, non-transferable use of the information. NO
 * WARRANTY of ANY KIND is provided. This heading must NOT be removed from
 * the file.
 *
 */

#include "nrf_drv_spi.h"
#include "app_util_platform.h"
#include "nrf_gpio.h"
#include "nrf_delay.h"
#include "boards.h"
#include "app_error.h"
#include <string.h>
#define NRF_LOG_MODULE_NAME "APP"
#include "nrf_log.h"
#include "nrf_log_ctrl.h"

// ### Workaround Section Start ###
#include "nrf_drv_ppi.h"
#include "nrf_drv_gpiote.h"
// ### Workaround Section End   ###

#define SPI_INSTANCE  0 /**< SPI instance index. */
static const nrf_drv_spi_t spi = NRF_DRV_SPI_INSTANCE(SPI_INSTANCE);  /**< SPI instance. */
static volatile bool spi_xfer_done;  /**< Flag used to indicate that SPI instance completed the transfer. */

#define TEST_STRING "N"

static uint8_t       m_tx_buf[1] = {'N'};           /**< TX buffer. */
static uint8_t       m_rx_buf[sizeof(TEST_STRING)+1];    /**< RX buffer. */
static const uint8_t m_length = sizeof(m_tx_buf);        /**< Transfer length. */

#define TEST_STRING_MULTI "Nordic"
static uint8_t       m_tx_buf_multi[] = TEST_STRING_MULTI;           /**< TX buffer. */
static uint8_t       m_rx_buf_multi[sizeof(TEST_STRING_MULTI)+1];    /**< RX buffer. */
static const uint8_t m_length_multi = sizeof(m_tx_buf_multi);        /**< Transfer length. */

// ### Workaround Section Start ###
static nrf_ppi_channel_t ppi_channel;
// ### Workaround Section End   ###

/**
 * @brief SPI user event handler.
 * @param event
 */
void spi_event_handler(nrf_drv_spi_evt_t const * p_event)
{    
    uint32_t err_code;
    
    switch(p_event->type)
    {
        case NRF_DRV_SPI_EVENT_DONE:
        
            // ### Workaround Section Start ###
        
            // Disable the PPI channel
            err_code = nrf_drv_ppi_channel_disable(ppi_channel);
            APP_ERROR_CHECK(err_code);

            // Disable a GPIOTE output pin task.
            nrf_drv_gpiote_in_event_disable(SPI_SCK_PIN);
        
            // ### Workaround Section End   ###
    spi_xfer_done = true;
    NRF_LOG_INFO("Transfer completed.\r\n");
    if (m_rx_buf[0] != 0)
    {
        NRF_LOG_INFO(" Received: \r\n");
        NRF_LOG_HEXDUMP_INFO(m_rx_buf, strlen((const char *)m_rx_buf));
    }
    default:
        break;
    }
}
    
//Dummy handler
void in_pin_handler( nrf_drv_gpiote_pin_t pin, nrf_gpiote_polarity_t action)
{
    //Do nothing
}

static void spi_send_recv(uint8_t * const p_tx_data,
                          uint8_t * const p_rx_data,
                          const uint16_t  len)
{
    //SEGGER_RTT_WriteString(0, "Send and Receive info \n");
    uint32_t err_code;

    if (len == 1){
            ret_code_t err_code;

            // Enable the PPI channel.
            err_code = nrf_drv_ppi_channel_enable(ppi_channel);
            APP_ERROR_CHECK(err_code);

            // Enable the GPIOTE output pin task.
            nrf_drv_gpiote_in_event_enable(SPI_SCK_PIN, false);

            //Start transfer of data
            err_code = nrf_drv_spi_transfer(&spi, p_tx_data, len, p_rx_data, len);
            APP_ERROR_CHECK(err_code);
            }
            else{
                // Start transfer.
                err_code = nrf_drv_spi_transfer(&spi, p_tx_data, len, p_rx_data, len);
                APP_ERROR_CHECK(err_code);
            }
}

int main(void)
{
    LEDS_CONFIGURE(BSP_LED_0_MASK);
    LEDS_OFF(BSP_LED_0_MASK);

    APP_ERROR_CHECK(NRF_LOG_INIT(NULL));

    NRF_LOG_INFO("SPI example\r\n");

    nrf_drv_spi_config_t spi_config = NRF_DRV_SPI_DEFAULT_CONFIG;
    spi_config.ss_pin   = SPI_SS_PIN;
    spi_config.miso_pin = SPI_MISO_PIN;
    spi_config.mosi_pin = SPI_MOSI_PIN;
    spi_config.sck_pin  = SPI_SCK_PIN;
    APP_ERROR_CHECK(nrf_drv_spi_init(&spi, &spi_config, spi_event_handler));
    
    // ### Workaround Section Start ###
    
     uint32_t err_code;
    
     NRF_SPIM_Type * p_spim = spi.p_registers;
    
    //Initialize GPIOTE and PPI drivers for the workaround
    
    err_code = nrf_drv_gpiote_init();
    APP_ERROR_CHECK(err_code);
    
    // Initializes the GPIOTE channel so that SCK toggles generates events
    nrf_drv_gpiote_in_config_t config = GPIOTE_CONFIG_IN_SENSE_TOGGLE(true);
            err_code = nrf_drv_gpiote_in_init(SPI_SCK_PIN, &config, in_pin_handler);
            APP_ERROR_CHECK(err_code); 
        
    err_code = nrf_drv_ppi_init();
    APP_ERROR_CHECK(err_code);
    
    // Allocate first unused PPI channel 
    err_code = nrf_drv_ppi_channel_alloc(&ppi_channel);
    APP_ERROR_CHECK(err_code);
    
    // Assign the PPI channel to the SCK pin
    err_code = nrf_drv_ppi_channel_assign(ppi_channel,
                                        nrf_drv_gpiote_in_event_addr_get(SPI_SCK_PIN),
                                        (uint32_t)&p_spim->TASKS_STOP);
    APP_ERROR_CHECK(err_code);


// ### Workaround Section End   ###    

    while (1)
    {
        // Reset rx buffer and transfer done flag
        memset(m_rx_buf, 0, m_length);
        memset(m_rx_buf_multi, 0, m_length_multi);
        spi_xfer_done = false;
 
        // Send single byte
        spi_send_recv(m_tx_buf, m_rx_buf, m_length);
    
        while (!spi_xfer_done)
        {
            __WFE();
        }
        
        // Send multiple bytes
        spi_xfer_done = false;
        spi_send_recv(m_tx_buf_multi, m_rx_buf_multi, m_length_multi);
       
        APP_ERROR_CHECK(err_code);
        
        while (!spi_xfer_done)
        {
            __WFE();
        }

        NRF_LOG_FLUSH();

        LEDS_INVERT(BSP_LED_0_MASK);
        nrf_delay_ms(200);
    }
}
